//
//  YHBrotherInfoVC.swift
//  galaxy
//
//  Created by edy on 2024/1/29.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

class YHBrotherInfoVC: YHBaseViewController {
    
    var brotherInfo:YHFamilyMember?
    
    var items:[[YHFormItemProtocol]] = [[YHFormItemProtocol]]()
    
    lazy var tableView: UITableView = {
        
        let tableView = UITableView(frame:.zero, style:.plain)

        if #available(iOS 11.0, *) {
            tableView.contentInsetAdjustmentBehavior = .never
        }
        tableView.showsVerticalScrollIndicator = false
        tableView.backgroundColor = .clear
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: "UITableViewCell")
        tableView.register(YHFormItemDoubleChoiceCell.self, forCellReuseIdentifier: YHFormItemDoubleChoiceCell.cellReuseIdentifier)
        tableView.register(YHFormItemInputTextCell.self, forCellReuseIdentifier: YHFormItemInputTextCell.cellReuseIdentifier)
        tableView.register(YHFormItemEnterDetailCell.self, forCellReuseIdentifier: YHFormItemEnterDetailCell.cellReuseIdentifier)
        tableView.register(YHFormItemTitleCell.self, forCellReuseIdentifier: YHFormItemTitleCell.cellReuseIdentifier)
        tableView.register(YHFormItemAddCell.self, forCellReuseIdentifier: YHFormItemAddCell.cellReuseIdentifier)
        tableView.register(YHFormItemSelectSheetCell.self, forCellReuseIdentifier: YHFormItemSelectSheetCell.cellReuseIdentifier)
        tableView.register(YHFormItemOnlyDoubleChoiceCell.self, forCellReuseIdentifier: YHFormItemOnlyDoubleChoiceCell.cellReuseIdentifier)
        
        return tableView
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        gk_navTitle = "兄弟姐妹信息填写".local
        createUI()
        loadBasicInfo()
    }
    
    func createUI() {
        
        view.addSubview(tableView);
        tableView.snp.makeConstraints { make in
            make.top.equalToSuperview().offset(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalToSuperview().offset(-100)
            make.left.equalToSuperview().offset(16)
            make.right.equalToSuperview().offset(-16)
        }
        loadBasicInfo()

    }
    
    func loadBasicInfo() {
        
        guard let brotherInfo = brotherInfo else { return }

        items.removeAll()
        
        // 兄弟姐妹信息
        let title0:YHFormItemProtocol = YHFormTitleItem(type: .brotherInfo)
        let item00:YHFormItemProtocol = YHFormDetailItem(type: .brotherName)
        let item01:YHFormItemProtocol = YHFormDetailItem(type: .birthday)
        let item02:YHFormItemProtocol = YHFormDetailItem(type: .birthNation)
        let item03:YHFormItemProtocol = YHFormDetailItem(type: .birthCity)
        let item04:YHFormItemProtocol = YHFormDetailItem(type: .gender)
        let item05:YHFormItemProtocol = YHFormDetailItem(type: .marriageState)
        let arr0:[YHFormItemProtocol] = [title0, item00, item01, item02, item03, item04, item05]
        
        // 职业信息
        let title1:YHFormItemProtocol = YHFormTitleItem(type: .occupationInfo)
        let item10:YHFormItemProtocol = YHFormDetailItem(type: .occupation)
        let arr1:[YHFormItemProtocol] = [title1, item10]
        
        // 居住信息
        let title2:YHFormItemProtocol = YHFormTitleItem(type: .liveInfo)
        let item20:YHFormItemProtocol = YHFormDetailItem(type: .liveNationOrArea)
        let arr2:[YHFormItemProtocol] = [title2, item20]
        
        // 香港身份证
        let title3:YHFormItemProtocol = YHFormTitleItem(type: .hkIdentityCardInfo)
        let item30:YHFormItemProtocol = YHFormDetailItem(type: .isHaveHkIdentityCard)
        let item31:YHFormItemProtocol = YHFormDetailItem(type: .hkIdentityCardNumber)
        let arr3:[YHFormItemProtocol] = [title3, item30, item31]
        
        items.append(contentsOf: [arr0, arr1, arr2, arr3, arr3])
        tableView.reloadData()
    
    }
    
    func getCellType(_ item: Any) ->HKFormItemCellType {
        if item is YHFormTitleItem {
            return .title
        }
        
        if item is YHFormDetailItem {
            let detailItem = item as! YHFormDetailItem
            if detailItem.type == .fatherName
               || detailItem.type == .motherName
               || detailItem.type == .nationOrArea
               || detailItem.type == .hkIdentityCardNumber
               || detailItem.type == .brotherName
            {
                return .inputText
            }
            if detailItem.type == .birthday
                || detailItem.type == .birthCity
                || detailItem.type == .marriageState
                || detailItem.type == .occupation
                || detailItem.type == .liveNationOrArea
                || detailItem.type == .gender
            {
                return .selectSheet
            }
            if detailItem.type == .birthNation
                || detailItem.type == .isHaveHkIdentityCard
            {
                return .twoChoice
            }
        }
        return .defaultType
    }
    
}

extension YHBrotherInfoVC : UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return items.count
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section >= items.count { return 0 }
        let arr = items[section]
        return arr.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        if indexPath.section >= items.count { return createDefaultCell(indexPath) }
        let arr:[YHFormItemProtocol] = items[indexPath.section]
        if indexPath.row >= arr.count {  return createDefaultCell(indexPath) }
        let item:YHFormItemProtocol = arr[indexPath.row]
        
        let cellType = getCellType(item)
        
        if cellType == .title { // 标题
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormItemTitleCell.cellReuseIdentifier, for: indexPath) as! YHFormItemTitleCell
            cell.titleLabel.text = item.getTitle()
            return cell
        }
        
        if cellType == .inputText { // 输入文字cell
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormItemInputTextCell.cellReuseIdentifier, for: indexPath) as! YHFormItemInputTextCell
            cell.title = item.getTitle()
            return cell
        }
        
        if cellType == .selectSheet { // 点击选择列表cell
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormItemSelectSheetCell.cellReuseIdentifier, for: indexPath) as! YHFormItemSelectSheetCell
            cell.title = item.getTitle()
            return cell
        }
        
        if cellType == .twoChoice { // 问答双项选择cell
            
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormItemDoubleChoiceCell.cellReuseIdentifier, for: indexPath) as! YHFormItemDoubleChoiceCell
            cell.title = item.getTitle()
            cell.answerArr = nil
            
            if item is YHFormDetailItem {
                let detailItem = item as! YHFormDetailItem
                if detailItem.type == .birthNation {
                    let answers = [YHFormChoiceItem(title: "国内".local, isSelect: true),
                                   YHFormChoiceItem(title: "国外".local, isSelect: false)]
                    cell.answerArr = answers
                } else if detailItem.type == .isHaveHkIdentityCard {
                    
                    let answers = [YHFormChoiceItem(title: "是".local, isSelect: true),
                                   YHFormChoiceItem(title: "否".local, isSelect: false)]
                    cell.answerArr = answers
                }
            }
            
            return cell
        }
        
        if cellType == .degreeDetailInfo { // 学位cell
            
            let cell = tableView.dequeueReusableCell(withIdentifier: YHFormItemDegreeInfoCell.cellReuseIdentifier, for: indexPath) as! YHFormItemDegreeInfoCell
            cell.title = item.getTitle()
            return cell
        }
        return createDefaultCell(indexPath)
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        
        if indexPath.section >= items.count { return  }
        let arr:[YHFormItemProtocol] = items[indexPath.section]
        if indexPath.row >= arr.count {  return }
        createCorner(cell: cell, arr: arr, indexPath: indexPath)
    }
    
    func createDefaultCell(_ indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "UITableViewCell", for: indexPath)
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        
        if indexPath.section >= items.count { return 0.0 }
        let arr:[YHFormItemProtocol] = items[indexPath.section]
        if indexPath.row >= arr.count {  return 0.0 }
        
        let item = arr[indexPath.row]
        
        if item is YHFormTitleItem { // 标题
            return 52.0
        }
        
        if item is YHFormAddItem {
            return 70.0
        }
        
        if item is YHFormDetailItem {
            return UITableView.automaticDimension
        }
        return 52.0
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        
        return UIView()
    }
    
    private func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> CGFloat {
        return 14.0
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {

        if !(0..<items.count).contains(indexPath.section) { return }
        let arr:[YHFormItemProtocol] = items[indexPath.section]
        if !(0..<arr.count).contains(indexPath.row) {  return  }
        let item = arr[indexPath.row]
        
        if item is YHFormDetailItem {
            let detailItem = item as! YHFormDetailItem
            if detailItem.type == .certificateValidDate ||
               detailItem.type == .certificateSignDate { // 届满日期 签发日期
     
                self.view?.endEditing(true)
                YHDatePickView.show(type: .yyyymmdd) { date in
                    let dataArr = date.components(separatedBy: YHDatePickView.separator)
                    print(date)
                    
                }
            }
            return
        }
    }
        
    func createCorner(cell:UITableViewCell, arr:Array<Any>, indexPath:IndexPath) {
        // 复用时需清理
        cell.layer.mask = nil
        // 设置每块section圆角
        if (indexPath.row == 0) {
            let corner = UIRectCorner(rawValue: UIRectCorner.topLeft.rawValue | UIRectCorner.topRight.rawValue)
            cell.createCorner(CGSizeMake(12.0, 12.0), corner)
            
        } else if (indexPath.row == arr.count-1) {
            let corner = UIRectCorner(rawValue: UIRectCorner.bottomLeft.rawValue | UIRectCorner.bottomRight.rawValue)
            cell.createCorner(CGSizeMake(12.0, 12.0), corner)
            
        } else {
            cell.layer.mask = nil
        }
    }
}

