//
//  YHSpouseInfoVC.swift
//  galaxy
//
//  Created by edy on 2024/1/31.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

@objc protocol YHSpouseInfoVCProtocol {
    
    @objc optional func updateStepView()
    @objc optional func saveInfo()
    
}

class YHSpouseInfoContainerVC: YHBaseViewController, YHSpouseInfoVCProtocol {

    let familyRequest:YHFamilyRequestViewModel = YHFamilyRequestViewModel()

    var orderId:Int = 0

    var spouse:YHFamilyMember? {
        didSet {
            initalInfoVC.spouse = spouse
            certificateVC.familyMember = spouse
            basicInfoVC.spouse = spouse
        }
    }
    
    var currentIndex = 0 {
        didSet {
            currentIndex %= self.children.count
            stepView.currentIndex = currentIndex
            for (i, vc) in self.children.enumerated() {
                vc.view.isHidden = (currentIndex != i)
                if currentIndex == i {
                    
                }
            }
        }
    }
    
    var stepView:YHStepView = {
        let step = YHStepView()
        return step
    }()
    
    var bottomView: YHSaveAndSubmitView = {
        return YHSaveAndSubmitView.createView()
    }()
        
    let initalInfoVC =  YHSpouseInitialInfoVC()
    let uploadVC = UIViewController()
    let certificateVC = YHCertificateInfoController()
    let basicInfoVC = YHSpouseBasicInfoVC()

    override func viewDidLoad() {
        super.viewDidLoad()
        createUI()
    }
    
    func createUI() {
        
        guard let spouse = spouse else { return }
        
        initalInfoVC.delegate = self
        certificateVC.delegate = self
        basicInfoVC.delegate = self
        
        updateStepView()

        currentIndex = 0
        
        view.addSubview(stepView)
        view.addSubview(bottomView)
        
        bottomView.submitBlock = {
            [weak self] in
            guard let self = self else { return }
            self.didClickSubmitBtn()
        }
        
        bottomView.saveBlock = {
            [weak self] in
            guard let self = self else { return }
            self.saveInfo()
        }
        
        stepView.snp.makeConstraints { make in
            make.top.equalTo(k_Height_NavigationtBarAndStatuBar)
            make.left.right.equalTo(view)
            make.height.equalTo(YHStepView.height)
        }
        
        bottomView.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.bottom.equalToSuperview().offset(-k_Height_safeAreaInsetsBottom())
            make.height.equalTo(YHSaveAndSubmitView.height)
        }
    }
    
    func didClickSubmitBtn() {
        currentIndex += 1;
        
    }
    
    func updateStepView() {
        
        guard let spouse = spouse else { return }
        
        for vc in self.children {
            vc.willMove(toParent: nil)
            vc.removeFromParent()
        }
        
        if spouse.isFollow() {
            
            self.addChild(initalInfoVC)
            self.addChild(uploadVC)
            self.addChild(certificateVC)
            self.addChild(basicInfoVC)
            
            self.view.addSubview(basicInfoVC.view)
            self.view.addSubview(certificateVC.view)
            self.view.addSubview(uploadVC.view)
            self.view.addSubview(initalInfoVC.view)
            
            stepView.dataSource = ["初始信息".local, "证件上传".local, "证件信息".local, "基本信息".local]
            stepView.block = {[weak self]  (index) in
                guard let self = self else { return }
                print("follow:%d", self.orderId)
            }
            
        } else {
            self.addChild(initalInfoVC)
            self.addChild(basicInfoVC)
            
            self.view.addSubview(basicInfoVC.view)
            self.view.addSubview(initalInfoVC.view)
            
            stepView.dataSource = ["初始信息".local, "基本信息".local]
            stepView.block = {[weak self]  (index) in
                guard let self = self else { return }
                print("follow:%d", self.orderId)

            }
        }
        self.view.bringSubviewToFront(stepView)
        self.view.bringSubviewToFront(bottomView)

    }
}

extension YHSpouseInfoContainerVC {
    func saveInfo() {
        
        guard  let spouse = spouse else { return }
        guard var info = spouse.toDictionary() else { return }
        
//        let certi = [ "cnIdentityCard":
//                        ["id":116871,"number":"421087199005010811","issue_at":"深圳","issue_date_start_at":"2017-01-01","issue_date_end_at":"2029-01-01","img_front":"","img_back":""],
//                        "passport":["id":116872,"number":"421087199005010811","issue_at":"深圳","issue_date_start_at":"2020-01-01","issue_date_end_at":"2023-01-01","img_front":"","img_back":"","passport_type":3],
//                        "hkMacaoPass":["id":118182,"number":"P103265(1)","issue_at":"SFAA","issue_date_start_at":"2024-01-31","issue_date_end_at":"2024-01-31","img_front":"","img_back":""]
//                        ]
//        
//        if let cer = info["certificates"] {
//            info["certificates"] = certi
//        }
        
        let cer = info["certificates"]
        let dict:[String: Any] = ["orderId":self.orderId,
                                  "relation":spouse.relationType.rawValue,
                                  "step":0,
                                  "next":false,
                                  "info":info]
        
        self.familyRequest.addOrSaveFamilyMember(params:dict) { [weak self] success, error in
            guard let self = self else { return }
            if success {
                
            }
        }
    }
}
